//
//  PSMLanPeer.h
//  BombSquad
//
//  Created by John Pannell on 2/15/10.
//  Copyright 2010 Positive Spin Media. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "PSMLanDefinitions.h"
#import "PSMLanProtocols.h"
#import "PSMLanSession.h"
#define kBufferSize                         1024

/*
 
 REMEMBER: a peer represents the session on the other end (i.e. its peerID and displayName are that of the session on the other end.
 
 */

@interface PSMLanPeer : NSObject {
    
    // delegate
    PSMLanSession<PSMLanPeerDelegate>   *session;

    // state
    PSMLanPeerConnectionState       connectionState;
    NSInteger                       errorCode;              // set if remote peer times out or cancels while waiting
    
    // representing the other end of the connection
    NSString                        *displayName;
    NSString                        *peerID;
    
    // comm
    NSNetService                    *remoteService;
    NSInputStream                   *inputStream;
    NSOutputStream                  *outputStream;
    NSMutableArray                  *packetQueue;
    NSData                          *readLeftover;
    NSData                          *writeLeftover;
    BOOL                            readReady;
    BOOL                            writeReady;
    BOOL                            acknowledged;
    
    NSTimer                         *requestTimer;
    
}

@property(readonly) PSMLanPeerConnectionState connectionState;
@property(readonly, retain) NSString *displayName;
@property(readonly, retain) NSString *peerID;
@property(readonly) NSNetService *remoteService;
@property(readwrite, retain) NSInputStream *inputStream;
@property(readwrite, retain) NSOutputStream *outputStream;
@property(readwrite) NSInteger errorCode;

// Creation
- (id)initWithNetService:(NSNetService *)ns session:(PSMLanSession *)object;

// connecting to the other end: client
- (void)initiateConnectionWithTimeout:(NSTimeInterval)interval;
- (void)cancelConnectionRequest;

// connecting to the other end: server
- (BOOL)acceptConnection:(NSError **)error;
- (void)denyConnection;

// disconnecting
- (void)disconnect;
- (void)kickOut; // force the remote peer to disconnect from all peers

// visibility
- (void)hide;

// usage
- (BOOL)sendData:(NSData *)data error:(NSError **)error;


@end
