//
//  PSMPacketCategories.m
//  BombSquad
//
//  Created by John Pannell on 2/16/10.
//  Copyright 2010 Positive Spin Media. All rights reserved.
//

#import "PSMPacketCategories.h"


@implementation NSArray(PacketSend)

- (NSData *)contentsForTransfer
{
    NSMutableData *ret = [NSMutableData data];
    for (NSData *item in self){
        // is it data?  if not, throw exception
        if(![item isKindOfClass:[NSData class]]){
            [NSException raise:kPSMPacketInvalidObjectException format:@"contentsForTransfer only supports instances of NSData"];
        }
        
        // create packet header (just the size of the data) and lump header and data into packet
        uint64_t dataSize[1];
        dataSize[0] = [item length];
        dataSize[0] = CFSwapInt64HostToLittle(dataSize[0]);
        [ret appendBytes:dataSize length:sizeof(uint64_t)];
        [ret appendBytes:[item bytes] length:[item length]];
    }
    
    return ret;
}

@end

@implementation NSData(PacketSplit)

- (NSArray *)splitTransferredPackets:(NSData **)leftover
{
    NSMutableArray *ret = [NSMutableArray array];
    const unsigned char *beginning = [self bytes];
    const unsigned char *offset = [self bytes];
    NSInteger bytesEnd = (NSInteger)offset + [self length];
    //NSLog(@"data length: %d", [self length]);
    
    while((NSInteger)offset < bytesEnd){
        // read header for size of data item
        //NSLog(@"thru");
        uint64_t dataSize[1];
        NSInteger dataSizeStart = offset - beginning;
        NSInteger dataStart = dataSizeStart + sizeof(uint64_t);
        
        NSRange headerRange = NSMakeRange(dataSizeStart, sizeof(uint64_t));
        [self getBytes:dataSize range:headerRange];
        dataSize[0] = CFSwapInt64LittleToHost(dataSize[0]);
        //NSLog(@"data start: %d, data size: %d", dataStart, (NSInteger)dataSize[0]);
        if((dataStart + dataSize[0] + (NSInteger)beginning) > bytesEnd){
            //NSLog(@"leftover!");
            // this item is not complete - file away leftovers for later
            NSInteger lengthOfRemainingData = [self length] - dataSizeStart;
            NSRange dataRange = NSMakeRange(dataSizeStart, lengthOfRemainingData);
            *leftover = [self subdataWithRange:dataRange];
            
            return ret;
        }
        
        NSRange dataRange = NSMakeRange(dataStart, dataSize[0]);
        NSData *parsedData = [self subdataWithRange:dataRange];
        
        // add data to array, advance the offset
        [ret addObject:parsedData];
        offset = offset + dataSize[0] + sizeof(uint64_t);
    }
    
    return ret;
}

@end

