//
//  TestGameController.m
//  BombSquad
//
//  Created by John Pannell on 2/25/10.
//  Copyright 2010 Positive Spin Media. All rights reserved.
//

#import "TestGameController.h"
#import "PSMLanDefinitions.h"
#import "PSMLanSession.h"
#import "PSMConnectStateValueTransformer.h"

@implementation TestGameController
@synthesize peerArray;

+ (void)initialize
{
    PSMConnectStateValueTransformer *vt = [[[PSMConnectStateValueTransformer alloc] init] autorelease];
    [NSValueTransformer setValueTransformer:vt forName:@"PSMConnectStateValueTransformer"];
}

- (id)init
{
    if(self = [super init]){
        peerArray = [[NSMutableArray alloc] init];
    }
    
    return self;
}

- (void)dealloc
{
    [peerArray release];
    [super dealloc];
}

#pragma mark -
#pragma mark UI Actions

- (IBAction)start:(id)sender
{
    // turn on network sharing
    lanSession = [[PSMLanSession alloc] initWithSessionID:@"TestLanApp" displayName:[myNameField stringValue] sessionMode:PSMLanSessionModePeer];
    [lanSession setDelegate:self];
    [lanSession setAvailable:YES];
    [lanSession setDataReceiveHandler:self withContext:nil];
    [sender setEnabled:NO];
}

- (IBAction)connect:(id)sender
{
    // start connection(s) with the selected peers
    NSArray *candidates = [peerController selectedObjects];
    for(NSDictionary *peerRep in candidates){
        [lanSession connectToPeer:[peerRep objectForKey:@"peerID"] withTimeout:40];
    }
}

- (IBAction)disconnect:(id)sender
{
    // disconnect the selected peers from the group
    NSArray *candidates = [peerController selectedObjects];
    for(NSDictionary *peerRep in candidates){
        [lanSession disconnectPeerFromAllPeers:[peerRep objectForKey:@"peerID"]];
    }
    
}

- (IBAction)dropOut:(id)sender
{
    // disconnect myself from other peers
    [lanSession disconnectFromAllPeers];
}

- (IBAction)send:(id)sender
{
    // send a message to the selected peers
    NSString *sendString = [NSString stringWithFormat:@"%@: %@\n", [lanSession displayName], [messageField stringValue]];
    NSLog(@"sending: \"%@\"", sendString);
    
    // send it out
    BOOL success = YES;
    NSError *error;
    NSArray *candidates = [peerController selectedObjects];
    for(NSDictionary *peerRep in candidates){
        if(![lanSession sendData:[sendString dataUsingEncoding:NSUTF8StringEncoding] toPeers:[NSArray arrayWithObject:[peerRep objectForKey:@"peerID"]] withDataMode:0 error:&error]){
            success = NO;
        }
    }
    
    // note in my personal log
    NSAttributedString *string;
    if(success){
        string = [[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@", sendString]] autorelease];
    } else {
        string = [[[NSAttributedString alloc] initWithFormat:@"send failure: %@", [error description]] autorelease];
    }
    
    NSTextStorage *storage = [messageLog textStorage];
    [storage beginEditing];
    [storage appendAttributedString:string];
    [storage endEditing];
}

#pragma mark -
#pragma mark Delegate Methods for PSMLanSession

// deleagate methods for GK/LAN Sessions
- (void)session:(id)session connectionWithPeerFailed:(NSString *)peerID withError:(NSError *)error
{
    
}

- (void)session:(id)session didFailWithError:(NSError *)error
{
    
}

- (void)session:(id)session didReceiveConnectionRequestFromPeer:(NSString *)peerID
{
    // I'm all about accepting
    [lanSession acceptConnectionFromPeer:peerID error:nil];
    NSLog(@"%@ accepted the connection request", [session displayName]);
}

- (void)session:(id)session peer:(NSString *)peerID didChangeState:(NSUInteger)state
{
    BOOL found = NO;
    // must find this peer in the collection
    for(NSMutableDictionary *peerRep in peerArray){
        if([[peerRep objectForKey:@"peerID"] isEqualToString:peerID]){
            found = YES;
            [peerRep setObject:[NSNumber numberWithInteger:state] forKey:@"connectionState"];
        }
    }
    
    if(!found){
        // new peer - create one and add to collection
        NSMutableDictionary *newPeerRep = [NSMutableDictionary dictionaryWithObjectsAndKeys:peerID, @"peerID", [session displayNameForPeer:peerID], @"displayName", [NSNumber numberWithInteger:state], @"connectionState", nil];
        NSLog(@"making new peer: %@", newPeerRep);
        [peerController addObject:newPeerRep];
    }
}


// getting data
- (void)receiveData:(NSData *)data fromPeer:(NSString *)peer inSession:(id)session context:(void *)context
{
    NSString *receiveString = [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease];
    NSAttributedString *string = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@", receiveString]];
    NSTextStorage *storage = [messageLog textStorage];
    
    [storage beginEditing];
    [storage appendAttributedString:string];
    [storage endEditing];
}



@end
