//
//  PSMPlayerProxy.m
//  BombSquad
//
//  Created by John Pannell on 2/10/10.
//  Copyright 2010 Positive Spin Media. All rights reserved.
//

#import "PSMPlayerProxy.h"
#import "PSMGameNode.h"
#import "PSMPlayerProtocols.h"
#import "NSInvocation+Serialization.h"

@interface PSMGameNode(ErrorHandling)
- (void)message:(NSString *)message toPlayer:(NSString *)playerID didFailWithError:(NSError *)error;
@end

@implementation PSMPlayerProxy
@synthesize playerID, displayName, hostSession, connectionType;

#pragma mark -
#pragma mark Creation/Destruction

- (id)initWithPlayerID:(NSString *)pid displayName:(NSString *)name session:(id)session connectionType:(PSMGameNodeConnectionType)type forNode:(PSMGameNode *)node
{

    playerID = [pid retain];
    displayName = [name retain];
    hostSession = session;
    connectionType = type;
    parentNode = node;
    
    return self;
}

- (void)dealloc
{
    [playerID release];
    [displayName release];
    
    [super dealloc];
}

#pragma mark -
#pragma mark Message Forwarding

-(void)forwardInvocation:(NSInvocation *)invocation
{
    // runtime did not find this method, giving me one last time to respond
    // I'm going to wrap it up as data and have my peer send it to the other end - reliably
    NSData *messageData = [invocation serializedData];
    NSError *error;
    if(![hostSession sendData:messageData toPeers:[NSArray arrayWithObject:playerID] withDataMode:0 error:&error]){
        NSLog(@"did not send");
        [parentNode message:NSStringFromSelector([invocation selector]) toPlayer:playerID didFailWithError:error];
    }
}

- (BOOL)respondsToSelector:(SEL)aSelector
{
    // there are a few that I will respond to :-)
    NSString *selector = NSStringFromSelector(aSelector);
    if([selector isEqualToString:@"displayName"])
        return YES;
    if([selector isEqualToString:@"playerID"])
        return YES;
    if([selector isEqualToString:@"hostSession"])
        return YES;
    if([selector isEqualToString:@"connectionType"])
        return YES;
    
    
    // use symmetry of nodes to determine if I can respond to this message
    id <PSMGameNodeDelegate,NSObject> target = [[parentNode delegate] invocationTargetForSelector:aSelector];
    return [target respondsToSelector:aSelector];
}

- (NSMethodSignature *)methodSignatureForSelector:(SEL)aSelector
{
    // must use symmetry of nodes to fetch target and method signature from this side of the connection
    NSObject <PSMGameNodeDelegate> *target = [[parentNode delegate] invocationTargetForSelector:aSelector];
    return [target methodSignatureForSelector:aSelector];
}

@end
