//
//  PKController.m
//  BombSquad
//
//  Created by John Pannell on 3/4/10.
//  Copyright 2010 Positive Spin Media. All rights reserved.
//

#import "PKController.h"
#import "PKConnectionStateValueTransformer.h"
#import "PSMGameNode.h"

@implementation PKController
@synthesize playerArray;

+ (void)initialize
{
    PKConnectionStateValueTransformer *vt = [[[PKConnectionStateValueTransformer alloc] init] autorelease];
    [NSValueTransformer setValueTransformer:vt forName:@"PSMConnectStateValueTransformer"];
}

- (id)init
{
    if(self = [super init]){
        playerArray = [[NSMutableArray alloc] init];
    }
    
    return self;
}

- (void)dealloc
{
    [playerArray release];
    [super dealloc];
}

#pragma mark -
#pragma mark UI Actions

- (IBAction)start:(id)sender
{
    // turn on network sharing
    gameNode = [[PSMGameNode alloc] initWithCommonIdentifier:@"TestPlayerKitApp" displayName:[myNameField stringValue] mode:PSMGameNodeModePeer];
    [gameNode setDelegate:self];
    [gameNode setAvailable:YES];
    [sender setEnabled:NO];
}

- (IBAction)connect:(id)sender
{
    // start connection(s) with the selected peers
    NSArray *candidates = [playerController selectedObjects];
    for(NSDictionary *playerRep in candidates){
        [gameNode connectToPlayer:[playerRep objectForKey:@"playerID"] withTimeout:40];
    }
}

- (IBAction)disconnect:(id)sender
{
    // disconnect the selected peers from the group
    NSArray *candidates = [playerController selectedObjects];
    for(NSDictionary *playerRep in candidates){
        [gameNode disconnectPlayerFromAllPlayers:[playerRep objectForKey:@"playerID"]];
    }
    
}

- (IBAction)dropOut:(id)sender
{
    // disconnect myself from other peers
    NSLog(@"drop out");
    [gameNode disconnectFromAllPlayers];
}

- (IBAction)send:(id)sender
{
    // send a message to the selected peers
    NSString *sendString = [NSString stringWithFormat:@"%@: %@\n", [gameNode displayName], [messageField stringValue]];
    
    // send it out
    NSArray *candidates = [playerController selectedObjects];
    for(NSDictionary *playerRep in candidates){
        [[gameNode playerForPlayerID:[playerRep objectForKey:@"playerID"]] displayMessage:sendString];
    }
    
    // note in my personal log
    [self displayMessage:sendString];
}

#pragma mark -
#pragma mark Delegate Methods for PSMGameNode

- (void)node:(id)node connectionWithPlayerFailed:(NSString *)playerID withError:(NSError *)error
{
    
}

- (void)node:(id)node didFailWithError:(NSError *)error
{
    
}

- (void)node:(id)node didReceiveConnectionRequestFromPlayer:(NSString *)playerID
{
    [gameNode acceptConnectionFromPlayer:playerID error:nil];
}

- (void)node:(id)node player:(NSString *)playerID didChangeState:(NSUInteger)state
{
    BOOL found = NO;
    // must find this player in the collection
    for(NSMutableDictionary *playerRep in playerArray){
        if([[playerRep objectForKey:@"playerID"] isEqualToString:playerID]){
            found = YES;
            [playerRep setObject:[NSNumber numberWithInteger:state] forKey:@"connectionState"];
        }
    }
    
    if(!found){
        // new player - create one and add to collection
        NSMutableDictionary *newPlayerRep = [NSMutableDictionary dictionaryWithObjectsAndKeys:playerID, @"playerID", [gameNode displayNameForPlayerID:playerID], @"displayName", [NSNumber numberWithInteger:state], @"connectionState", nil];
        [playerController addObject:newPlayerRep];
    }
}

- (id)invocationTargetForSelector:(SEL)selector
{
    return self;
}

- (id)forwardingTargetForSelector:(SEL)selector
{
    return self;
}

- (void)message:(NSString *)message toPlayer:(NSString *)playerID didFailWithError:(NSError *)error
{
    [self displayMessage:[NSString stringWithFormat:@"The previous message failed to send to %@", [gameNode displayNameForPlayerID:playerID]]];
}


#pragma mark -
#pragma mark P2P Communication

- (void)displayMessage:(NSString *)message
{
    NSAttributedString *string = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@", message]];
    NSTextStorage *storage = [messageLog textStorage];
    
    [storage beginEditing];
    [storage appendAttributedString:string];
    [storage endEditing];
}


@end
